@@ -6,15 +6,15 @@ from os import environ
 
 APP_ROOT = Path(__file__).resolve().parents[0]
 
-if 'GENE_NORM_EB_PROD' in environ:
-    LOG_FN = '/tmp/gene.log'
+if "GENE_NORM_EB_PROD" in environ:
+    LOG_FN = "/tmp/gene.log"
 else:
-    LOG_FN = 'gene.log'
+    LOG_FN = "gene.log"
 
 logging.basicConfig(
     filename=LOG_FN,
-    format='[%(asctime)s] - %(name)s - %(levelname)s : %(message)s')
-logger = logging.getLogger('gene')
+    format="[%(asctime)s] - %(name)s - %(levelname)s : %(message)s")
+logger = logging.getLogger("gene")
 logger.setLevel(logging.DEBUG)
 logger.handlers = []
 
@@ -25,13 +25,14 @@ logging.getLogger("python_jsonschema_objects").setLevel(logging.INFO)
 logging.getLogger("biocommons.seqrepo.seqaliasdb.seqaliasdb").setLevel(logging.INFO)  # noqa: E501
 logging.getLogger("biocommons.seqrepo.fastadir.fastadir").setLevel(logging.INFO)  # noqa: E501
 
-if 'GENE_NORM_EB_PROD' in environ:
+if "GENE_NORM_EB_PROD" in environ:
     ch = logging.StreamHandler()
     ch.setLevel(logging.INFO)
     logger.addHandler(ch)
 
 
-SEQREPO_DATA_PATH = '/usr/local/share/seqrepo/latest'
+SEQREPO_DATA_PATH = environ.get("SEQREPO_DATA_PATH",
+                                "/usr/local/share/seqrepo/latest")
 
 
 class DownloadException(Exception):
@@ -53,16 +54,16 @@ SOURCES = {source.value.lower(): source.value
 XREF_SOURCES = {src.lower() for src in SourceName.__members__}
 
 # use to fetch source name from schema based on concept id namespace
-# e.g. {'hgnc': 'HGNC'}
+# e.g. {"hgnc": "HGNC"}
 PREFIX_LOOKUP = {v.value: SourceName[k].value
                  for k, v in NamespacePrefix.__members__.items()
                  if k in SourceName.__members__.keys()}
 
 # use to generate namespace prefix from source ID value
-# e.g. {'ensg': 'ensembl'}
+# e.g. {"ensg": "ensembl"}
 NAMESPACE_LOOKUP = {v.value.lower(): NamespacePrefix[k].value
                     for k, v in SourceIDAfterNamespace.__members__.items()
-                    if v.value != ''}
+                    if v.value != ""}
 
 from gene.etl import NCBI, HGNC, Ensembl  # noqa: E402, F401
 # used to get source class name from string
