@@ -1,13 +1,14 @@
 """This module provides methods for handling queries."""
 import re
-from typing import List, Dict, Set
+from typing import List, Dict, Set, Any, TypeVar, Callable, Optional
 from urllib.parse import quote
 from .version import __version__
 from gene import NAMESPACE_LOOKUP, PREFIX_LOOKUP, ITEM_TYPES
 from gene.database import Database
-from gene.schemas import Gene, SourceMeta, MatchType, SourceName, \
+from gene.schemas import BaseGene, Gene, SourceMeta, MatchType, SourceName, \
     ServiceMeta, SourcePriority, NormalizeService, SearchService, \
-    GeneTypeFieldName
+    GeneTypeFieldName, UnmergedNormalizationService, MatchesNormalized, \
+    BaseNormalizationService
 from ga4gh.vrsatile.pydantic.vrsatile_models import GeneDescriptor, Extension
 from botocore.exceptions import ClientError
 from boto3.dynamodb.conditions import Key
@@ -15,6 +16,9 @@ from datetime import datetime
 from gene import logger
 
 
+NormService = TypeVar("NormService", bound=BaseNormalizationService)
+
+
 class InvalidParameterException(Exception):
     """Exception for invalid parameter args provided by the user."""
 
@@ -73,6 +77,23 @@ class QueryHandler:
             except ClientError as e:
                 logger.error(e.response['Error']['Message'])
 
+    @staticmethod
+    def _cast_location_ints(record: Dict) -> Dict:
+        """Ensure Locations are formatted correctly -- interval start and end need to
+        be recast to ints from how they're structured in DynamoDB
+
+        :param Dict record: original record
+        :return: record with corrected locations attributes, if applicable
+        """
+        if 'locations' in record:
+            for loc in record['locations']:
+                if loc['interval']['type'] == "SequenceInterval":
+                    loc['interval']['start']['value'] = \
+                        int(loc['interval']['start']['value'])
+                    loc['interval']['end']['value'] = \
+                        int(loc['interval']['end']['value'])
+        return record
+
     def add_record(self,
                    response: Dict[str, Dict],
                    item: Dict,
@@ -88,13 +109,7 @@ class QueryHandler:
         """
         del item['label_and_type']
         # DynamoDB Numbers get converted to Decimal
-        if 'locations' in item:
-            for loc in item['locations']:
-                if loc['interval']['type'] == "SequenceInterval":
-                    loc['interval']['start']['value'] = \
-                        int(loc['interval']['start']['value'])
-                    loc['interval']['end']['value'] = \
-                        int(loc['interval']['end']['value'])
+        item = self._cast_location_ints(item)
         item["match_type"] = match_type
         gene = Gene(**item)
         src_name = item['src_name']
@@ -311,36 +326,62 @@ class QueryHandler:
         resp['service_meta_'] = self._get_service_meta()
         return SearchService(**resp)
 
-    def _add_merged_meta(self, response: Dict) -> Dict:
+    def _add_merged_meta(self, response: NormalizeService) -> NormalizeService:
         """Add source metadata to response object.
 
         :param Dict response: in-progress response object
         :return: completed response object.
         """
         sources_meta = {}
-        gene_descr = response['gene_descriptor']
-        ids = [gene_descr['gene_id']] + gene_descr.get('xrefs', [])
+        gene_descr = response.gene_descriptor
+        xrefs = gene_descr.xrefs or []  # type: ignore
+        ids = [gene_descr.gene_id] + xrefs  # type: ignore
         for concept_id in ids:
             prefix = concept_id.split(':')[0]
             src_name = PREFIX_LOOKUP[prefix.lower()]
             if src_name not in sources_meta:
                 sources_meta[src_name] = self.fetch_meta(src_name)
-        response['source_meta_'] = sources_meta
+        response.source_meta_ = sources_meta
         return response
 
-    def add_gene_descriptor(self, response, record, match_type,
-                            possible_concepts=[]) -> Dict:
+    def _add_alt_matches(self, response: NormService, record: Dict,
+                         possible_concepts: List[str]) -> NormService:
+        """Add alternate matches warning to response object
+
+        :param NormService response: in-progress response object
+        :param Dict record: normalized record
+        :param List[str] possible_concepts: other possible matches
+        :return: updated response object
+        """
+        norm_concepts = set()
+        for concept_id in possible_concepts:
+            r = self.db.get_record_by_id(concept_id, True)
+            if r:
+                merge_ref = r.get("merge_ref")
+                if merge_ref:
+                    norm_concepts.add(merge_ref)
+        norm_concepts = norm_concepts - {record["concept_id"]}
+        if norm_concepts:
+            response.warnings.append({
+                "multiple_normalized_concepts_found": list(norm_concepts)
+            })
+        return response
+
+    def add_gene_descriptor(
+        self, response: NormalizeService, record: Dict, match_type: MatchType,
+        possible_concepts: Optional[List[str]] = None
+    ) -> NormalizeService:
         """Add gene descriptor to response.
 
         :param Dict response: Response object
         :param Dict record: Gene record
         :param MatchType match_type: query's match type
-        :param list possible_concepts: List of other normalized concepts
-            found
+        :param Optional[List[str]] possible_concepts: List of other normalized
+            concepts found
         :return: Response with gene descriptor
         """
         params = {
-            "id": f"normalize.gene:{quote(response['query'])}",
+            "id": f"normalize.gene:{quote(response.query)}",
             "label": record["symbol"],
             "gene_id": record["concept_id"]
         }
@@ -382,7 +423,7 @@ class QueryHandler:
             gene_type = record.get("gene_type")
             if gene_type:
                 extensions.append(Extension(
-                    name=GeneTypeFieldName[record["src_name"].upper()],
+                    name=GeneTypeFieldName[record["src_name"].upper()].value,
                     value=gene_type
                 ))
         else:
@@ -399,25 +440,12 @@ class QueryHandler:
 
         # add warnings
         if possible_concepts:
-            norm_concepts = set()
-            for concept_id in possible_concepts:
-                r = self.db.get_record_by_id(concept_id, True)
-                if r:
-                    merge_ref = r.get("merge_ref")
-                    if merge_ref:
-                        norm_concepts.add(merge_ref)
-            norm_concepts = norm_concepts - {record["concept_id"]}
-            if norm_concepts:
-                response["warnings"].append(
-                    {
-                        "multiple_normalized_concepts_found":
-                            list(norm_concepts)
-                    }
-                )
-        response["gene_descriptor"] = \
-            GeneDescriptor(**params).dict(exclude_none=True)
+            response = self._add_alt_matches(response, record,
+                                             possible_concepts)
+
+        response.gene_descriptor = GeneDescriptor(**params)
         response = self._add_merged_meta(response)
-        response["match_type"] = match_type
+        response.match_type = match_type
         return response
 
     @staticmethod
@@ -445,94 +473,174 @@ class QueryHandler:
         response['match_type'] = MatchType.NO_MATCH
         return response
 
+    def _prepare_normalized_response(self, query: str) -> Dict[str, Any]:
+        """Provide base response object for normalize endpoints.
+
+        :param str query: user-provided query
+        :return: basic normalization response boilerplate
+        """
+        return {
+            "query": query,
+            "match_type": MatchType.NO_MATCH,
+            "warnings": self.emit_warnings(query),
+            "service_meta_": ServiceMeta(
+                version=__version__,
+                response_datetime=str(datetime.now()))
+        }
+
     def normalize(self, query: str) -> NormalizeService:
         """Return normalized concept for query.
 
         :param str query: String to find normalized concept for
         :return: Normalized gene concept
         """
-        response = {
-            "query": query,
-            "warnings": self.emit_warnings(query),
-            "service_meta_": self._get_service_meta()
-        }
+        response = NormalizeService(**self._prepare_normalized_response(query))
+        return self._perform_normalized_lookup(response, query,
+                                               self.add_gene_descriptor)
 
-        if query == '':
-            response['match_type'] = MatchType.NO_MATCH
-            return NormalizeService(**response)
+    def _resolve_merge(
+        self, response: NormService, record: Dict, match_type: MatchType,
+        callback: Callable, possible_concepts: Optional[List[str]] = None
+    ) -> NormService:
+        """Given a record, return the corresponding normalized record
+
+        :param NormalizationService response: in-progress response object
+        :param Dict record: record to retrieve normalized concept for
+        :param MatchType match_type: type of match that returned these records
+        :param Callable callback: response constructor method
+        :param Optional[List[str]] possible_concepts: alternate possible matches
+        :return: Normalized response object
+        """
+        merge_ref = record.get("merge_ref")
+        if merge_ref:
+            # follow merge_ref
+            merge = self.db.get_record_by_id(merge_ref, False, True)
+            if merge is None:
+                query = response.query
+                logger.error(
+                    f"Merge ref lookup failed for ref {record['merge_ref']} "
+                    f"in record {record['concept_id']} from query `{query}`"
+                )
+                return response
+            else:
+                return callback(response, merge, match_type, possible_concepts)
+        else:
+            # record is sole member of concept group
+            return callback(response, record, match_type, possible_concepts)
+
+    def _get_matches_by_type(self, query: str, match_type: str) -> List[Dict]:
+        """Get matches list for match tier.
+        :param str query: user-provided query
+        :param str match_type: keyword of match type to check
+        :return: List of records matching the query and match level
+        """
+        matching_refs = self.db.get_records_by_type(query, match_type)
+        matching_records = [self.db.get_record_by_id(m["concept_id"], False)
+                            for m in matching_refs]
+        return sorted(matching_records, key=self._record_order)  # type: ignore
+
+    def _perform_normalized_lookup(
+        self, response: NormService, query: str, response_builder: Callable
+    ) -> NormService:
+        """Retrieve normalized concept, for use in normalization endpoints
+        :param NormService response: in-progress response object
+        :param str query: user-provided query
+        :param Callable response_builder: response constructor callback method
+        :return: completed service response object
+        """
+        if query == "":
+            return response
         query_str = query.lower().strip()
 
         # check merged concept ID match
-        record = self.db.get_record_by_id(query_str, case_sensitive=False,
-                                          merge=True)
+        record = self.db.get_record_by_id(query_str, case_sensitive=False, merge=True)
         if record:
-            response = self.add_gene_descriptor(
-                response, record, MatchType.CONCEPT_ID)
-            return NormalizeService(**response)
+            return response_builder(response, record, MatchType.CONCEPT_ID)
 
         # check concept ID match
         record = self.db.get_record_by_id(query_str, case_sensitive=False)
         if record:
-            merge_ref = record.get('merge_ref')
-            if not merge_ref:
-                response = self.add_gene_descriptor(
-                    response, record, MatchType.CONCEPT_ID)
-                return NormalizeService(**response)
-            merge = self.db.get_record_by_id(merge_ref,
-                                             case_sensitive=False,
-                                             merge=True)
-            if merge is None:
-                response = self._handle_failed_merge_ref(
-                    record, response, query_str)
-                return NormalizeService(**response)
-            else:
-                response = self.add_gene_descriptor(
-                    response, merge, MatchType.CONCEPT_ID)
-                return NormalizeService(**response)
+            return self._resolve_merge(response, record, MatchType.CONCEPT_ID,
+                                       response_builder)
 
-        # check other match types
-        matching_records = None
         for match_type in ITEM_TYPES.values():
             # get matches list for match tier
             matching_refs = self.db.get_records_by_type(query_str, match_type)
             matching_records = \
                 [self.db.get_record_by_id(m['concept_id'], False)
                  for m in matching_refs]
-            matching_records.sort(key=self._record_order)
+            matching_records.sort(key=self._record_order)  # type: ignore
 
             if len(matching_refs) > 1:
                 possible_concepts = \
                     [ref["concept_id"] for ref in matching_refs]
             else:
-                possible_concepts = []
+                possible_concepts = None
 
             # attempt merge ref resolution until successful
             for match in matching_records:
-                record = self.db.get_record_by_id(match['concept_id'], False)
+                assert match is not None
+                record = self.db.get_record_by_id(match["concept_id"], False)
                 if record:
-                    merge_ref = record.get('merge_ref')
-                    if not merge_ref:
-                        response = self.add_gene_descriptor(
-                            response, record,
-                            MatchType[match_type.upper()],
-                            possible_concepts
-                        )
-                        return NormalizeService(**response)
-                    merge = self.db.get_record_by_id(record['merge_ref'],
-                                                     case_sensitive=False,
-                                                     merge=True)
-                    if merge is None:
-                        response = self._handle_failed_merge_ref(
-                            record, response, query_str)
-                        return NormalizeService(**response)
-                    else:
-                        response = self.add_gene_descriptor(
-                            response, merge,
-                            MatchType[match_type.upper()],
-                            possible_concepts
-                        )
-                        return NormalizeService(**response)
+                    match_type_value = MatchType[match_type.upper()]
+                    return self._resolve_merge(
+                        response, record, match_type_value,
+                        response_builder, possible_concepts
+                    )
+        return response
 
-        if not matching_records:
-            response['match_type'] = MatchType.NO_MATCH
-        return NormalizeService(**response)
+    def _add_normalized_records(
+        self, response: UnmergedNormalizationService, normalized_record: Dict,
+        match_type: MatchType, possible_concepts: Optional[List[str]] = None
+    ) -> UnmergedNormalizationService:
+        """Add individual records to unmerged normalize response.
+
+        :param UnmergedNormalizationService response: in-progress response
+        :param Dict normalized_record: record associated with normalized concept,
+            either merged or single identity
+        :param MatchType match_type: type of match achieved
+        :param Optional[List[str]] possible_concepts: other possible results
+        :return: Completed response object
+        """
+        response.match_type = match_type
+        response.normalized_concept_id = normalized_record["concept_id"]
+        if normalized_record["item_type"] == "identity":
+            record_source = SourceName[normalized_record["src_name"].upper()]
+            response.source_matches[record_source] = MatchesNormalized(
+                records=[BaseGene(**self._cast_location_ints(normalized_record))],
+                source_meta_=self.fetch_meta(record_source.value)
+            )
+        else:
+            concept_ids = [normalized_record["concept_id"]] + \
+                normalized_record.get("xrefs", [])
+            for concept_id in concept_ids:
+                record = self.db.get_record_by_id(concept_id, case_sensitive=False)
+                if not record:
+                    continue
+                record_source = SourceName[record["src_name"].upper()]
+                gene = BaseGene(**self._cast_location_ints(record))
+                if record_source in response.source_matches:
+                    response.source_matches[record_source].records.append(gene)
+                else:
+                    response.source_matches[record_source] = MatchesNormalized(
+                        records=[gene],
+                        source_meta_=self.fetch_meta(record_source.value)
+                    )
+        if possible_concepts:
+            response = self._add_alt_matches(response, normalized_record,
+                                             possible_concepts)
+        return response
+
+    def normalize_unmerged(self, query: str) -> UnmergedNormalizationService:
+        """Return all source records under the normalized concept for the
+        provided query string.
+
+        :param str query: string to search against
+        :return: Normalized response object
+        """
+        response = UnmergedNormalizationService(
+            source_matches={},
+            **self._prepare_normalized_response(query)
+        )
+        return self._perform_normalized_lookup(response, query,
+                                               self._add_normalized_records)
